'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.findOwnerClassPath = findOwnerClassPath;
exports.isConstructor = isConstructor;
exports.isEventAttribute = isEventAttribute;
exports.isPureMemberExpressionWithThis = isPureMemberExpressionWithThis;
exports.isPureCallExpressionWithThis = isPureCallExpressionWithThis;
exports.isReactClass = isReactClass;
function findOwnerClassPath(path) {
  return path.findParent(function (path) {
    return path.isClassExpression();
  });
}

function isConstructor(path, t) {
  return t.isIdentifier(path.get('key').node, { name: 'constructor' });
}

// E.g. onClick={}
function isEventAttribute(path, t) {
  var propPrefix = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 'on';

  var ret = false;

  if (!t.isJSXIdentifier(path.get('name')) || !t.isJSXExpressionContainer(path.get('value'))) {
    return ret;
  }

  if (Array.isArray(propPrefix)) {
    ret = propPrefix.some(function (prefix) {
      var rName = new RegExp('^' + prefix);

      return rName.exec(path.get('name').node.name);
    });
  } else {
    var rName = new RegExp('^' + propPrefix);

    if (rName.exec(path.get('name').node.name)) {
      ret = true;
    }
  }

  return ret;
}

// E.g. onClick={this.handleClick}
function isPureMemberExpressionWithThis(path, t) {
  return t.isMemberExpression(path) && t.isThisExpression(path.get('object').node) && t.isIdentifier(path.get('property').node);
}

// E.g. onClick={this.handleClick(item)}
function isPureCallExpressionWithThis(path, t) {
  return t.isCallExpression(path) && isPureMemberExpressionWithThis(path.get('callee'), t);
}

function isPathReactClass(path) {
  if (path.matchesPattern('React.Component') || path.matchesPattern('React.PureComponent')) {
    return true;
  }

  if (path.node && (path.node.name === 'Component' || path.node.name === 'PureComponent')) {
    return true;
  }

  return false;
}

function isReactClass(superClass, scope) {
  var ret = false;

  if (isPathReactClass(superClass)) {
    ret = true;
  } else if (superClass.node.name) {
    var className = superClass.node.name,
        binding = scope.getBinding(className);

    if (!binding) {
      ret = false;
    } else {
      superClass = binding.path.get('superClass');

      if (isPathReactClass(superClass)) {
        ret = true;
      }
    }
  }

  return ret;
}