'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _factory = require('./factory');

var _helpers = require('./helpers');

function plugin(_ref) {
  var t = _ref.types;

  return {
    visitor: {
      Program: function Program(programPath, state) {
        programPath.traverse({
          ClassExpression: {
            enter: function enter(path) {
              var superClass = path.get('superClass'),
                  classMethodPaths = path.get('body').get('body');

              if (!superClass.node || !(0, _helpers.isReactClass)(superClass, path.scope)) {
                return;
              }
              this.addedEventNames = [];
              if (!classMethodPaths.some(function (classMethodPath) {
                return (0, _helpers.isConstructor)(classMethodPath, t);
              })) {
                path.get('body').node.body.unshift((0, _factory.buildConstructor)(t));
              }
            },
            exit: function exit() {
              delete this.addedEventNames;
            }
          },
          JSXAttribute: function JSXAttribute(path) {
            var eventName = void 0,
                classPath = void 0;

            if (!(0, _helpers.isEventAttribute)(path, t, this.state.opts.propPrefix)) {
              return;
            }
            if (this.state.opts.advanced === true && (0, _helpers.isPureCallExpressionWithThis)(path.get('value').get('expression'), t)) {
              var handler = path.get('value').get('expression').get('callee').get('property').node.name,
                  nodeArgs = path.get('value').get('expression').node.arguments;

              path.get('value').get('expression').replaceWith((0, _factory.buildArrowFunction)(t, handler, nodeArgs));

              return;
            }
            if (!(0, _helpers.isPureMemberExpressionWithThis)(path.get('value').get('expression'), t)) {
              return;
            }

            eventName = path.get('value').get('expression').get('property').node.name;
            if (!this.addedEventNames || ~this.addedEventNames.indexOf(eventName)) {
              return;
            }
            classPath = (0, _helpers.findOwnerClassPath)(path);
            classPath.traverse({
              ClassMethod: function ClassMethod(path) {
                if ((0, _helpers.isConstructor)(path, t) && (0, _helpers.findOwnerClassPath)(path) === classPath) {
                  path.get('body').pushContainer('body', (0, _factory.buildBindingStatement)(t, eventName));
                  this.addedEventNames.push(eventName);
                }
              }
            }, { addedEventNames: this.addedEventNames });
          }
        }, { addedEventNames: null, state: state });
      }
    }
  };
}

exports.default = plugin;